--[[ Define your Highlight Keyword here! ]]--

highlightString = "Falcon"

--[[

-= XChat2 G15 mod 1.0.1 =-

Changes/Additions by Severin "Falcon" Lochinger
http://www.hisdarkness.de
eMail: falcon@hisdarkness.de
FalconX on Qnet and IRCnet

All Credits for version 1.0 go to the original author! See below!

Changes since 1.0:
- Added Highlight String: Define at the beginning of this .lua (highlightString = " ")
- Changed function RefreshScreen: I will be no longer bringing the applet to front on G15 LCD, unless there is an highlight message.
- Changed function XChat_Message: Added string.find to determine whether the incoming text-message contains an highlight.

Future plans: 
- Adding function so the script reads out XChats default highlight Strings. For now the highlightString has to be defined manually above.

]]--


--[[

 -= XChat2 G15 mod 1.0 =.
 
by Bastian Pflieger <wb@illogical.de>
http://www.illogical.de
wb_ in #pro @ Qnet

Feel free to hack / edit / modify this script in any way you wish.
Please, share the results.

This is G15 mod is based on two components:
- Plugin .dll which handles connection to LCD Display and Xchat2 plugin details.
- This lua script which handles the logic above and decides what you see on the LCD.

This seperation has a few good points:
- You can edit the mod by yourself.
- Saves me from doing all in plain C/C++ (ok, C++ has powerful STL which would make it less painful).
- Easier to understand and modify.
- LUA allows rapid development and is very powerful / fast.

The script have to defines two entry points:
function XChat_Message(msg)
  - Gets called for every incoming message, hooks PRIVMSG in XChat.
function XChat_OnUpdate(isButton1Down, isButton2Down, isButton3Down, isButton4Down)
  - Gets called every 100ms with current button status.
These two have to be defined, I currently do not check if they exists. If they are not defined XChat crash immediately.

 The script can call these exposed C functions:
 LCD_Connect()
   - establish connection to LCD Display, call once.
     Returns isConnected, height, width, bpp, number of buttons of LCD.
 LCD_Disconnect()
   - no need to call, called automatically called when XChat is closed or plugin unloaded.
 LCD_UpdateBitmap(table)
   - Uses data from table to draw on LCD. table should be an array with pixelNr => PixelStatus.
     PixelStatus is a number: <127 pixel is off, >= 128 pixel is on.
     For the G15 the table have to contain 6880 entries.
 XChat_Print(string)
   - Prints string in current XChat window (it is not send to other users).
   
--]]

-- I have choosen 5x5 pixel for one character, but you have to calculate with 6x6 as you need a space to the right and at the bottom.
-- This results in 26 Characters horizontal and 7 lines on the G15 (160x43 resolution).
-- 26 * 6 = 156, the last four pixel columns are used for status info (scrollbar, mode display).

MATRIX_CHAR_UNKNOWN = { 128, 128, 128, 128, 128,
                        128, 127, 127, 127, 128,
                        128, 127, 127, 127, 128,
                        128, 127, 127, 127, 128,
                        128, 128, 128, 128, 128 }

MATRIX_CHAR_A = { 127, 128, 128, 128, 127,
                  128, 127, 127, 127, 128,
                  128, 128, 128, 128, 128,
                  128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128 }
                  
MATRIX_CHAR_B = { 128, 128, 128, 128, 127,
                  128, 127, 127, 127, 128,
                  128, 128, 128, 128, 127,
                  128, 127, 127, 127, 128,
                  128, 128, 128, 128, 127 }

MATRIX_CHAR_C = { 127, 128, 128, 128, 128,
                  128, 127, 127, 127, 127,
                  128, 127, 127, 127, 127,
                  128, 127, 127, 127, 127,
                  127, 128, 128, 128, 128 }

MATRIX_CHAR_D = { 128, 128, 128, 128, 127,
                  128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  128, 128, 128, 128, 127 }

MATRIX_CHAR_E = { 128, 128, 128, 128, 128,
                  128, 127, 127, 127, 127,
                  128, 128, 128, 128, 127,
                  128, 127, 127, 127, 127,
                  128, 128, 128, 128, 128 }

MATRIX_CHAR_F = { 128, 128, 128, 128, 128,
                  128, 127, 127, 127, 127,
                  128, 128, 128, 128, 127,
                  128, 127, 127, 127, 127,
                  128, 127, 127, 127, 127 }

MATRIX_CHAR_G = { 127, 128, 128, 128, 128,
                  128, 127, 127, 127, 127,
                  128, 127, 128, 128, 128,
                  128, 127, 127, 127, 128,
                  127, 128, 128, 128, 127 }

MATRIX_CHAR_H = { 128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  128, 128, 128, 128, 128,
                  128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128 }

MATRIX_CHAR_I = { 128, 128, 128, 128, 128,
                  127, 127, 128, 127, 127,
                  127, 127, 128, 127, 127,
                  127, 127, 128, 127, 127,
                  128, 128, 128, 128, 128 }

MATRIX_CHAR_J = { 127, 127, 127, 127, 128,
                  127, 127, 127, 127, 128,
                  127, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  127, 128, 128, 128, 127 }
                  
MATRIX_CHAR_K = { 128, 127, 127, 128, 127,
                  128, 127, 128, 127, 127,
                  128, 128, 127, 127, 127,
                  128, 127, 128, 127, 127,
                  128, 127, 127, 128, 127 }

MATRIX_CHAR_L = { 128, 127, 127, 127, 127,
                  128, 127, 127, 127, 127,
                  128, 127, 127, 127, 127,
                  128, 127, 127, 127, 127,
                  128, 128, 128, 128, 128 }

MATRIX_CHAR_M = { 127, 128, 128, 128, 127,
                  128, 127, 128, 127, 128,
                  128, 127, 128, 127, 128,
                  128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128 }

MATRIX_CHAR_N = { 128, 127, 127, 127, 128,
                  128, 128, 127, 127, 128,
                  128, 127, 128, 127, 128,
                  128, 127, 127, 128, 128,
                  128, 127, 127, 127, 128 }

MATRIX_CHAR_O = { 127, 128, 128, 128, 127,
                  128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  127, 128, 128, 128, 127 }

MATRIX_CHAR_P = { 128, 128, 128, 128, 127,
                  128, 127, 127, 127, 128,
                  128, 128, 128, 128, 127,
                  128, 127, 127, 127, 127,
                  128, 127, 127, 127, 127 }

MATRIX_CHAR_Q = { 127, 128, 128, 128, 127,
                  128, 127, 127, 127, 128,
                  128, 127, 128, 127, 128,
                  128, 127, 127, 128, 127,
                  127, 128, 128, 127, 128 }

MATRIX_CHAR_R = { 128, 128, 128, 128, 127,
                  128, 127, 127, 127, 128,
                  128, 128, 128, 128, 127,
                  128, 127, 128, 127, 127,
                  128, 127, 127, 128, 127 }

MATRIX_CHAR_S = { 127, 128, 128, 128, 128,
                  128, 127, 127, 127, 127,
                  127, 128, 128, 128, 127,
                  127, 127, 127, 127, 128,
                  128, 128, 128, 128, 127 }

MATRIX_CHAR_T = { 128, 128, 128, 128, 128,
                  127, 127, 128, 127, 127,
                  127, 127, 128, 127, 127,
                  127, 127, 128, 127, 127,
                  127, 127, 128, 127, 127 }

MATRIX_CHAR_U = { 128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  127, 128, 128, 128, 127 }                  
                  
MATRIX_CHAR_V = { 128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  127, 128, 127, 128, 127,
                  127, 128, 127, 128, 127,
                  127, 127, 128, 127, 127 }
                  
MATRIX_CHAR_W = { 128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  128, 127, 128, 127, 128,
                  127, 128, 128, 128, 127 }                  
                  
MATRIX_CHAR_X = { 128, 127, 127, 127, 128,
                  127, 128, 127, 128, 127,
                  127, 127, 128, 127, 127,
                  127, 128, 127, 128, 127,
                  128, 127, 127, 127, 128 }

MATRIX_CHAR_Y = { 128, 127, 127, 127, 128,
                  128, 127, 127, 127, 128,
                  127, 128, 127, 128, 127,
                  127, 127, 128, 127, 127,
                  127, 127, 128, 127, 127 }

MATRIX_CHAR_Z = { 128, 128, 128, 128, 128,
                  127, 127, 127, 128, 127,
                  127, 127, 128, 127, 127,
                  127, 128, 127, 127, 127,
                  128, 128, 128, 128, 128 }

MATRIX_CHAR_1 = { 127, 127, 127, 128, 127,
                  127, 127, 128, 128, 127,
                  127, 128, 127, 128, 127,
                  127, 127, 127, 128, 127,
                  127, 127, 127, 128, 127 }

MATRIX_CHAR_2 = { 128, 128, 128, 128, 127,
                  127, 127, 127, 127, 128,
                  127, 128, 128, 128, 127,
                  128, 127, 127, 127, 127,
                  128, 128, 128, 128, 128 }

MATRIX_CHAR_3 = { 128, 128, 128, 128, 127,
                  127, 127, 127, 127, 128,
                  127, 128, 128, 128, 127,
                  127, 127, 127, 127, 128,
                  128, 128, 128, 128, 127 }

MATRIX_CHAR_4 = { 127, 127, 128, 127, 127,
                  127, 128, 127, 127, 127,
                  128, 127, 127, 128, 127,
                  128, 128, 128, 128, 128,
                  127, 127, 127, 128, 127 }

MATRIX_CHAR_5 = { 127, 128, 128, 128, 128,
                  128, 127, 127, 127, 127,
                  128, 128, 128, 128, 128,
                  127, 127, 127, 127, 128,
                  128, 128, 128, 128, 128 }

MATRIX_CHAR_6 = { 127, 128, 128, 128, 128,
                  128, 127, 127, 127, 127,
                  128, 128, 128, 128, 128,
                  128, 127, 127, 127, 128,
                  128, 128, 128, 128, 128 }

MATRIX_CHAR_7 = { 128, 128, 128, 128, 128,
                  127, 127, 127, 127, 128,
                  127, 127, 127, 128, 127,
                  127, 127, 128, 127, 127,
                  127, 128, 127, 127, 127 }

MATRIX_CHAR_8 = { 127, 128, 128, 128, 127,
                  128, 127, 127, 127, 128,
                  127, 128, 128, 128, 127,
                  128, 127, 127, 127, 128,
                  127, 128, 128, 128, 127 }

MATRIX_CHAR_9 = { 127, 128, 128, 128, 127,
                  128, 127, 127, 127, 128,
                  127, 128, 128, 128, 128,
                  127, 127, 127, 127, 128,
                  128, 128, 128, 128, 127 }
                  
MATRIX_CHAR_0 = { 127, 128, 128, 128, 127,
                  128, 127, 127, 128, 128,
                  128, 127, 128, 127, 128,
                  128, 128, 127, 127, 128,
                  127, 128, 128, 128, 127 }

MATRIX_CHAR_FS = { 127, 127, 127, 127, 127,
                   127, 127, 127, 127, 127,
                   127, 127, 127, 127, 127,
                   127, 128, 128, 127, 127,
                   127, 128, 128, 127, 127 }                   

MATRIX_CHAR_EX = { 127, 128, 128, 127, 127,
                   127, 128, 128, 127, 127,
                   127, 128, 128, 127, 127,
                   127, 127, 127, 127, 127,
                   127, 128, 128, 127, 127 }                   

MATRIX_CHAR_LEFT_PARAENTHESIS = { 127, 128, 128, 127, 127,
                                  128, 127, 127, 127, 127,
                                  128, 127, 127, 127, 127,
                                  128, 127, 127, 127, 127,
                                  127, 128, 128, 127, 127 }                   
                        
MATRIX_CHAR_RIGHT_PARAENTHESIS = { 127, 127, 128, 128, 127,
                                   127, 127, 127, 127, 128,
                                   127, 127, 127, 127, 128,
                                   127, 127, 127, 127, 128,
                                   127, 127, 128, 128, 127 }                   
                   
MATRIX_CHAR_NUMBERSIGN= { 127, 128, 127, 128, 127,
                          128, 128, 128, 128, 128,
                          127, 128, 127, 128, 127,
                          128, 128, 128, 128, 128,
                          127, 128, 127, 128, 127 }   
                           
MATRIX_CHAR_SPACE = { 127, 127, 127, 127, 127,
                      127, 127, 127, 127, 127,
                      127, 127, 127, 127, 127,
                      127, 127, 127, 127, 127,
                      127, 127, 127, 127, 127 }   

MATRIX_CHAR_COLON = { 127, 128, 128, 127, 127,
                      127, 128, 128, 127, 127,
                      127, 127, 127, 127, 127,
                      127, 128, 128, 127, 127,
                      127, 128, 128, 127, 127 }   
                      
MATRIX_CHAR_GREATER_THAN = { 127, 128, 127, 127, 127,
                             127, 127, 128, 127, 127,
                             127, 127, 127, 128, 127,
                             127, 127, 128, 127, 127,
                             127, 128, 127, 127, 127 }   

MATRIX_CHAR_LESSER_THAN = { 127, 127, 127, 128, 127,
                            127, 127, 128, 127, 127,
                            127, 128, 127, 127, 127,
                            127, 127, 128, 127, 127,
                            127, 127, 127, 128, 127 }   
                   
MATRIX_CHAR_UNDERSCORE = { 127, 127, 127, 127, 127,
                           127, 127, 127, 127, 127,
                           127, 127, 127, 127, 127,
                           127, 127, 127, 127, 127,
                           128, 128, 128, 128, 128 }      

MATRIX_CHAR_EQUALSIGN = { 127, 127, 127, 127, 127,
                          128, 128, 128, 128, 128,
                          127, 127, 127, 127, 127,
                          128, 128, 128, 128, 128,
                          127, 127, 127, 127, 127 }                           

MATRIX_CHAR_SEPERATOR = { 127, 127, 128, 127, 127,
                          127, 127, 128, 127, 127,
                          127, 127, 128, 127, 127,
                          127, 127, 128, 127, 127,
                          127, 127, 128, 127, 127 }                           

MATRIX_CHAR_QUESTIONMARK = { 127, 128, 128, 127, 127,
                             127, 127, 127, 128, 127,
                             127, 127, 128, 127, 127,
                             127, 127, 127, 127, 127,
                             127, 127, 128, 127, 127 }                           

MATRIX_CHAR_INVERTED_COMMA = { 127, 128, 127, 127, 127,
                               127, 127, 128, 127, 127,
                               127, 127, 127, 127, 127,
                               127, 127, 127, 127, 127,
                               127, 127, 127, 127, 127 }                           

MATRIX_CHAR_APOSTROPHE = { 127, 127, 128, 127, 127,
                           127, 127, 128, 127, 127,
                           127, 127, 127, 127, 127,
                           127, 127, 127, 127, 127,
                           127, 127, 127, 127, 127 }                           

MATRIX_CHAR_COMMA = { 127, 127, 127, 127, 127,
                      127, 127, 127, 127, 127,
                      127, 127, 127, 127, 127,
                      127, 127, 128, 127, 127,
                      127, 128, 127, 127, 127 }                           
                      
MATRIX_CHAR_SEMICOLON = { 127, 127, 127, 127, 127,
                          127, 127, 128, 127, 127,
                          127, 127, 127, 127, 127,
                          127, 127, 128, 127, 127,
                          127, 128, 127, 127, 127 }                           

MATRIX_CHAR_MINUS = { 127, 127, 127, 127, 127,
                      127, 127, 127, 127, 127,
                      128, 128, 128, 128, 128,
                      127, 127, 127, 127, 127,
                      127, 127, 127, 127, 127 }                           

MATRIX_CHAR_PLUS = { 127, 127, 128, 127, 127,
                     127, 127, 128, 127, 127,
                     128, 128, 128, 128, 128,
                     127, 127, 128, 127, 127,
                     127, 127, 128, 127, 127 }                           
                     
MATRIX_CHAR_MUL = { 127, 127, 128, 127, 127,
                    127, 128, 128, 128, 127,
                    128, 128, 128, 128, 128,
                    127, 128, 128, 128, 127,
                    127, 127, 128, 127, 127 }                     

MATRIX_CHAR_EYES = { 127, 127, 128, 127, 127,
                     127, 128, 127, 128, 127,
                     128, 127, 127, 127, 128,
                     127, 127, 127, 127, 127,
                     127, 127, 127, 127, 127 }                           

MATRIX_CHAR_SLASH = { 127, 127, 127, 127, 128,
                      127, 127, 127, 128, 127,
                      127, 127, 128, 127, 127,
                      127, 128, 127, 127, 127,
                      128, 127, 127, 127, 127 }

MATRIX_CHAR_BACKSLASH = { 128, 127, 127, 127, 127,
                          127, 128, 127, 127, 127,
                          127, 127, 128, 127, 127,
                          127, 127, 127, 128, 127,
                          127, 127, 127, 127, 128 }
                  
MATRIX_CHAR_AT = { 127, 128, 128, 128, 127,
                   128, 127, 127, 127, 128,
                   128, 127, 128, 128, 128,
                   128, 127, 127, 127, 127,
                   127, 128, 128, 128, 127 }                      
                   
MATRIX_CHAR_TILDE = { 127, 127, 127, 127, 127,
                      127, 128, 127, 127, 127,
                      128, 127, 128, 127, 128,
                      127, 127, 127, 128, 127,
                      127, 127, 127, 127, 127 }
                      
MATRIX_CHAR_DOUBLEQUOTE = { 127, 128, 127, 128, 127,
                            127, 128, 127, 128, 127,
                            127, 127, 127, 127, 127,
                            127, 127, 127, 127, 127,
                            127, 127, 127, 127, 127 }                      
                        
MATRIX_CHAR_PARAGRAPH = { 127, 128, 128, 128, 127,
                          128, 127, 128, 127, 127,
                          127, 128, 127, 128, 127,
                          127, 127, 128, 127, 128,
                          127, 128, 128, 128, 127 }

MATRIX_CHAR_DOLLAR = { 127, 128, 128, 128, 128,
                       128, 127, 128, 127, 127,
                       127, 128, 128, 128, 127,
                       127, 127, 128, 127, 128,
                       128, 128, 128, 128, 127 }
                       
MATRIX_CHAR_PROCENT = { 128, 128, 127, 127, 128,
                        128, 128, 127, 128, 127,
                        127, 127, 128, 127, 127,
                        127, 128, 127, 128, 128,
                        128, 127, 127, 128, 128 }                       
                        
MATRIX_CHAR_AMP = { 127, 128, 128, 127, 127,
                    128, 127, 127, 128, 127,
                    127, 128, 128, 127, 127,
                    128, 127, 127, 128, 127,
                    128, 128, 128, 127, 128 }                        

MATRIX_CHAR_LEFT_BRACKET = { 127, 128, 128, 128, 127,
                             127, 128, 127, 127, 127,
                             127, 128, 127, 127, 127,
                             127, 128, 127, 127, 127,
                             127, 128, 128, 128, 127 }
                        
MATRIX_CHAR_RIGHT_BRACKET = { 127, 128, 128, 128, 127,
                              127, 127, 127, 128, 127,
                              127, 127, 127, 128, 127,
                              127, 127, 127, 128, 127,
                              127, 128, 128, 128, 127 }                  

MATRIX_CHAR_LEFT_BRACE = { 127, 127, 128, 128, 127,
                           127, 128, 127, 127, 127,
                           128, 128, 127, 127, 127,
                           127, 128, 127, 127, 127,
                           127, 127, 128, 128, 127 }
                  
MATRIX_CHAR_RIGHT_BRACE = { 127, 128, 128, 127, 127,
                            127, 127, 127, 128, 127,
                            127, 127, 127, 128, 128,
                            127, 127, 127, 128, 127,
                            127, 128, 128, 127, 127 }
                  
MATRIX_CHAR_GERMAN_UE = { 127, 128, 127, 128, 127,
                          127, 127, 127, 127, 127,
                          128, 127, 127, 127, 128,
                          128, 127, 127, 127, 128,
                          127, 128, 128, 128, 127 }                           
                          
MATRIX_CHAR_GERMAN_AE = { 127, 128, 127, 128, 127,
                          127, 128, 128, 128, 127,
                          128, 127, 127, 127, 128,
                          128, 128, 128, 128, 128,
                          128, 127, 127, 127, 128 }
                          
MATRIX_CHAR_GERMAN_OE = { 127, 128, 127, 128, 127,
                          127, 128, 128, 128, 127,
                          128, 127, 127, 127, 128,
                          128, 127, 127, 127, 128,
                          127, 128, 128, 128, 127 }
                          
MATRIX_CHAR_GERMAN_SS = { 128, 128, 127, 127, 127,
                          128, 128, 128, 127, 127,
                          128, 127, 127, 128, 127,
                          128, 128, 128, 128, 127,
                          128, 127, 127, 127, 127 }
           
MATRIX_CHAR_NEW_MESSAGE = { 128, 128, 128, 128, 128,
                            128, 128, 128, 128, 128,
                            128, 128, 128, 128, 128,
                            128, 128, 128, 128, 128,
                            128, 128, 128, 128, 128 }
                      
           
local CHAR_TO_MATRIX = {
 A = MATRIX_CHAR_A,
 a = MATRIX_CHAR_A,
 B = MATRIX_CHAR_B,
 b = MATRIX_CHAR_B,
 C = MATRIX_CHAR_C,
 c = MATRIX_CHAR_C,
 D = MATRIX_CHAR_D,
 d = MATRIX_CHAR_D,
 E = MATRIX_CHAR_E,
 e = MATRIX_CHAR_E,
 F = MATRIX_CHAR_F,
 f = MATRIX_CHAR_F,
 G = MATRIX_CHAR_G,
 g = MATRIX_CHAR_G,
 H = MATRIX_CHAR_H,
 h = MATRIX_CHAR_H,
 I = MATRIX_CHAR_I,
 i = MATRIX_CHAR_I,
 J = MATRIX_CHAR_J,
 j = MATRIX_CHAR_J,
 K = MATRIX_CHAR_K,
 k = MATRIX_CHAR_K,
 L = MATRIX_CHAR_L,
 l = MATRIX_CHAR_L,
 M = MATRIX_CHAR_M,
 m = MATRIX_CHAR_M,
 N = MATRIX_CHAR_N,
 n = MATRIX_CHAR_N,
 O = MATRIX_CHAR_O,
 o = MATRIX_CHAR_O,
 P = MATRIX_CHAR_P,
 p = MATRIX_CHAR_P,
 Q = MATRIX_CHAR_Q,
 q = MATRIX_CHAR_Q,
 R = MATRIX_CHAR_R,
 r = MATRIX_CHAR_R,
 S = MATRIX_CHAR_S,
 s = MATRIX_CHAR_S,
 T = MATRIX_CHAR_T,
 t = MATRIX_CHAR_T,
 U = MATRIX_CHAR_U,
 u = MATRIX_CHAR_U,
 V = MATRIX_CHAR_V,
 v = MATRIX_CHAR_V,
 W = MATRIX_CHAR_W,
 w = MATRIX_CHAR_W,
 X = MATRIX_CHAR_X,
 x = MATRIX_CHAR_X,
 Y = MATRIX_CHAR_Y,
 y = MATRIX_CHAR_Y,
 Z = MATRIX_CHAR_Z,
 z = MATRIX_CHAR_Z,
 ["1"] = MATRIX_CHAR_1,
 ["2"] = MATRIX_CHAR_2,
 ["3"] = MATRIX_CHAR_3,
 ["4"] = MATRIX_CHAR_4,
 ["5"] = MATRIX_CHAR_5,
 ["6"] = MATRIX_CHAR_6,
 ["7"] = MATRIX_CHAR_7,
 ["8"] = MATRIX_CHAR_8,
 ["9"] = MATRIX_CHAR_9,
 ["0"] = MATRIX_CHAR_0,
 ["."] = MATRIX_CHAR_FS,
 ["!"] = MATRIX_CHAR_EX,
 ["("] = MATRIX_CHAR_LEFT_PARAENTHESIS,
 [")"] = MATRIX_CHAR_RIGHT_PARAENTHESIS, 
 [" "] = MATRIX_CHAR_SPACE, 
 ["#"] = MATRIX_CHAR_NUMBERSIGN, 
 ["<"] = MATRIX_CHAR_LESSER_THAN, 
 [">"] = MATRIX_CHAR_GREATER_THAN, 
 [":"] = MATRIX_CHAR_COLON, 
 ["_"] = MATRIX_CHAR_UNDERSCORE,
 ["="] = MATRIX_CHAR_EQUALSIGN,
 ["|"] = MATRIX_CHAR_SEPERATOR,
 ["?"] = MATRIX_CHAR_QUESTIONMARK,
 ["`"] = MATRIX_CHAR_INVERTED_COMMA,
 ["'"] = MATRIX_CHAR_APOSTROPHE,
 [","] = MATRIX_CHAR_COMMA,
 [";"] = MATRIX_CHAR_SEMICOLON,
 ["-"] = MATRIX_CHAR_MINUS,
 ["+"] = MATRIX_CHAR_PLUS,
 ["*"] = MATRIX_CHAR_MUL,
 ["^"] = MATRIX_CHAR_EYES,
 ["/"] = MATRIX_CHAR_SLASH,
 ["\\"] = MATRIX_CHAR_BACKSLASH,
 ["@"] = MATRIX_CHAR_AT,
 ["\""] = MATRIX_CHAR_DOUBLEQUOTE,
 ["\194\167"] = MATRIX_CHAR_PARAGRAPH,
 ["$"] = MATRIX_CHAR_DOLLAR,
 ["%"] = MATRIX_CHAR_PROCENT,
 ["&"] = MATRIX_CHAR_AMP,
 ["["] = MATRIX_CHAR_LEFT_BRACKET,
 ["]"] = MATRIX_CHAR_RIGHT_BRACKET,
 ["{"] = MATRIX_CHAR_LEFT_BRACE,
 ["}"] = MATRIX_CHAR_RIGHT_BRACE,
 ["~"] = MATRIX_CHAR_TILDE,
 ["\240\130\130\130"] = MATRIX_CHAR_NEW_MESSAGE,
 ["\195\188"] = MATRIX_CHAR_GERMAN_UE, -- 
 ["\195\156"] = MATRIX_CHAR_GERMAN_UE, -- 
 ["\195\164"] = MATRIX_CHAR_GERMAN_AE, -- 
 ["\195\132"] = MATRIX_CHAR_GERMAN_AE, -- 
 ["\195\182"] = MATRIX_CHAR_GERMAN_OE, -- 
 ["\195\150"] = MATRIX_CHAR_GERMAN_OE, -- 
 ["\195\159"] = MATRIX_CHAR_GERMAN_SS, -- 
}

local MAX_CACHE_LINES = 100
local EMPTY_LINE = { " " }
local CHANNEL_ALL = "All Messages"
local MODE_AUTO = 1
local MODE_MANUEL = 0


-- Holds messages for every channel
local channels = { 
  [CHANNEL_ALL] = {},
}
table.insert(channels, CHANNEL_ALL)

local config = {
  channel = CHANNEL_ALL,
  channelIndex = 1,
  mode = MODE_AUTO,
  scrollPos = 0,
  statusMessageTimer = -1,
  inputThrottle = -1,
}

local LcdMatrix = {}
-- fill in pixels
for i = 1, 6880 do
  table.insert(LcdMatrix, 127)
end

-- turns every pixel off
function LcdClearMatrix()
  for i = 1, 6880 do
    LcdMatrix[i] = 127
  end
end
-- prints up to 26 chars on a given line
function AddLineToMatrix(line, utf8Chunks)
  local yTop = (line - 1) * 6 * 160 -- determine Yoffset for line
  local xOffset = 0 -- write from left to right
  
  -- zero out
  for i = 1, 800 do
    LcdMatrix[yTop + i] = 127
  end

  for i = 1, table.getn(utf8Chunks) do
    local charMatrix = MATRIX_CHAR_UNKNOWN
    if CHAR_TO_MATRIX[utf8Chunks[i]] then
      charMatrix = CHAR_TO_MATRIX[utf8Chunks[i]]
    end
    -- print char, copy char matrix to LCD Matrix
    for yy = 0, 4 do
      for xx = 1, 5 do
        LcdMatrix[(yTop + (yy * 160)) + xx + xOffset] = charMatrix[(yy * 5) + xx]
      end
    end
    xOffset = xOffset + 6
  end
end                  

--  divide a string into its UTF8 chunks,
-- after that store up to 26 chunks per table entry (buffer)
function AddUtf8ToBuffer(s, buffer)
  i = 1
  local utf8Vec = {}
  while i <= string.len(s) do
    if string.byte(s, i) >= 240 then
      table.insert(utf8Vec, string.sub(s, i, i + 3))
      i = i + 4
    elseif string.byte(s, i) >= 224 then
      table.insert(utf8Vec, string.sub(s, i, i + 2))
      i = i + 3
    elseif string.byte(s, i) >= 192 then
      table.insert(utf8Vec, string.sub(s, i, i + 1))
      i = i + 2
    else
      -- it is a ascii char
      table.insert(utf8Vec, string.lower(string.sub(s, i, i)))
      i = i + 1
    end   
  end
  
  local lines = math.ceil(table.getn(utf8Vec) / 26)
  for i = 1, lines do
    local startIndex = ((i - 1) * 26)
    local chunks = {}
    for j = 1, 26 do
      if (startIndex + j) <= table.getn(utf8Vec) then
        table.insert(chunks, utf8Vec[startIndex + j])
      end
    end
    table.insert(buffer, chunks)    
  end
end


function PrintBuffer(buffer)
  for i = 0, 6 do
    if (config.scrollPos - i) >= 1 then
      AddLineToMatrix(7 - i, buffer[config.scrollPos - i])
    end
  end
end

function PrintStatus(buffer)
  if config.mode == MODE_AUTO then
    LcdMatrix[158] = 128
    LcdMatrix[159] = 128
    LcdMatrix[160] = 128
    
    LcdMatrix[160 + 158] = 128
    LcdMatrix[160 * 2] = 128
    
    LcdMatrix[160 * 2 + 158] = 128
    LcdMatrix[160 * 2 + 159] = 128
    LcdMatrix[160 * 3] = 128
    
    LcdMatrix[160 * 3 + 158] = 128
    LcdMatrix[160 * 4] = 128
    
    LcdMatrix[160 * 4 + 158] = 128
    LcdMatrix[160 * 5] = 128   
  end
  
  -- ScrollBar
  local scrollBarYStart = 12
  local scrollBarYEnd = 42
  local scrollBarYSize = scrollBarYEnd - scrollBarYStart
  local lines = table.getn(buffer) - 6
  
  if table.getn(buffer) > 7 then
    LcdMatrix[160 * (scrollBarYStart - 1)] = 128
    LcdMatrix[160 * (scrollBarYStart - 1) - 1] = 128
    LcdMatrix[160 * (scrollBarYStart - 1) - 2] = 128
    LcdMatrix[160 * (scrollBarYEnd + 1)] = 128
    LcdMatrix[160 * (scrollBarYEnd + 1) - 1] = 128
    LcdMatrix[160 * (scrollBarYEnd + 1) - 2] = 128

    for y = scrollBarYStart, scrollBarYEnd do
      LcdMatrix[160 * y] = 128
      LcdMatrix[160 * y - 1] = 128
      LcdMatrix[160 * y - 2] = 128
    end
    local lineYScale = scrollBarYSize / lines
    local YOffset = math.floor(lineYScale * (config.scrollPos - 6))
   
    if lineYScale > 1 then
      lineYScale = math.floor(lineYScale)
      for i = 0, (lineYScale) do
        LcdMatrix[160 * (scrollBarYStart + YOffset - i) - 1] = 127
      end
    else
      LcdMatrix[160 * (scrollBarYStart + YOffset) - 1] = 127
    end
    
  end  
  
end

function PrintStatusMessage()
  if config.statusMessage then
    AddLineToMatrix(2, EMPTY_LINE)
    AddLineToMatrix(3, config.statusMessage[1])
    AddLineToMatrix(4, config.statusMessage[2])
    AddLineToMatrix(5, EMPTY_LINE)
  end
end

function RefreshScreen(buffer)
  LcdClearMatrix()
  PrintBuffer(buffer)
  PrintStatusMessage()
  PrintStatus(buffer)

  -- Ask whether there is an Highlight or not, before output, defining applet priority in case there is
  if highlight_msg then
	LCD_UpdateBitmap(LcdMatrix, LGLCD_PRIORITY_ALERT)
  else
	LCD_UpdateBitmap(LcdMatrix)
	end
	
end

-- One entry point, called from XChat plugin for every incoming message (hooks PRIV)
function XChat_Message(msg)
  -- extract information from chatmessage
  local _, _, channel, text = string.find(msg, "PRIVMSG (.-) :(.+)")
  local _, _, emoteText = string.find(text, ".ACTION (.+).")
  local _, _, nick = string.find(msg, ":(.-)!.*")
  


-- Check if there is a Highlight keyword in the msg. Returns true if there is. Returns NIL when not

  highlight_msg = string.find(msg, highlightString)
  
  

  if nick and channel and text then
    -- if new channel, create buffer and integer index with channel name
    if not channels[channel] then
      channels[channel] = {}
      table.insert(channels, channel)
    end
    
    -- save scroll status in case we add multiple lines
    local isScrollAtBottom = (config.scrollPos == table.getn(channels[config.channel]))
    
    -- decide how to format output 
    if emoteText then
      AddUtf8ToBuffer(string.format("\240\130\130\130%s %s %s", channel, nick, emoteText), channels[channel])
      AddUtf8ToBuffer(string.format("\240\130\130\130%s %s %s", channel, nick, emoteText), channels[CHANNEL_ALL])
    else
      AddUtf8ToBuffer(string.format("\240\130\130\130%s <%s>: %s", channel, nick, text), channels[channel])
      AddUtf8ToBuffer(string.format("\240\130\130\130%s <%s>: %s", channel, nick, text), channels[CHANNEL_ALL])
    end    
    
    -- keep only 100 LCD lines in channel buffer
    while (table.getn(channels[channel] ) > MAX_CACHE_LINES) do
      table.remove(channels[channel], 1)
    end
    while (table.getn(channels[CHANNEL_ALL] ) > MAX_CACHE_LINES) do
      table.remove(channels[CHANNEL_ALL] , 1)
    end
    
    -- in auto mode we have to display this channel NOW
    if config.mode == MODE_AUTO then
      config.channel = channel
      config.scrollPos = table.getn(channels[channel])
      -- does not set channelindex, so if user press next channel button he is still at his old location! is this good or bad?
      RefreshScreen(channels[channel])
    elseif channel == config.channel or config.channel == CHANNEL_ALL then
      -- refresh if it is our current channel
      if isScrollAtBottom then
        config.scrollPos = table.getn(channels[config.channel])
      end
      RefreshScreen(channels[config.channel])
    end
  end
end

-- called every 100ms from XChat
function XChat_OnUpdate(isButton1Down, isButton2Down, isButton3Down, isButton4Down)
  config.statusMessageTimer = config.statusMessageTimer - 1
  config.inputThrottle = config.inputThrottle - 1

  if config.statusMessageTimer == 0 then
    -- delete status message
    config.statusMessage = nil
    RefreshScreen(channels[config.channel])
  end
  
  if config.inputThrottle <  0 then
    -- switch between auto (jump to active channels automatically) and manuel mode (stay in this channel and switch manuel)
    if isButton4Down and isButton3Down then
      if config.mode == MODE_AUTO then
        config.mode = MODE_MANUEL
        RefreshScreen(channels[config.channel])
      else
        config.mode = MODE_AUTO
        RefreshScreen(channels[config.channel])    
      end
      config.inputThrottle = 5
    elseif isButton2Down or isButton1Down then
      -- button1 and button2 used to navigate through channels
      if isButton1Down then 
        config.channelIndex = config.channelIndex - 1
      else
        config.channelIndex = config.channelIndex + 1
      end
      if config.channelIndex < 1 then
        config.channelIndex = table.getn(channels)
      elseif config.channelIndex > table.getn(channels) then
        config.channelIndex = 1
      end    
      -- assign current channel name
      config.channel = channels[config.channelIndex]
      config.mode = MODE_MANUEL   
      config.statusMessage = {}
      AddUtf8ToBuffer(" \240\130\130\130  New Channel:           \240\130\130\130\240\130\130\130 " .. config.channel, config.statusMessage)
      config.statusMessageTimer = 40
      config.scrollPos = table.getn(channels[config.channel])
      -- show new channel
      RefreshScreen(channels[config.channel])
      config.inputThrottle = 5
    elseif isButton4Down and config.scrollPos > 7 and table.getn(channels[config.channel]) > 7 then
      config.scrollPos = config.scrollPos - 1
      config.mode = MODE_MANUEL
      RefreshScreen(channels[config.channel])
      config.inputThrottle = 1
    elseif isButton3Down and config.scrollPos < table.getn(channels[config.channel]) then
      config.scrollPos = config.scrollPos + 1
      config.mode = MODE_MANUEL
      RefreshScreen(channels[config.channel])
      config.inputThrottle = 1
    end
  end
end

-- Init LCD and show welcome message
LCD_Connect()
AddUtf8ToBuffer("  \240\130\130\130\240\130\130\130 XChat2 G15 Plugin \240\130\130\130\240\130\130\130", channels[CHANNEL_ALL])
AddUtf8ToBuffer(" ", channels[CHANNEL_ALL])
AddUtf8ToBuffer("  http://www.illogical.de", channels[CHANNEL_ALL])
AddUtf8ToBuffer(" ", channels[CHANNEL_ALL])
config.scrollPos = table.getn(channels[CHANNEL_ALL])
RefreshScreen(channels[CHANNEL_ALL])
